using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using System.Threading.Tasks;
using avalonia_with_entity.Models;
using Microsoft.EntityFrameworkCore;
using ReactiveUI;

namespace avalonia_with_entity.ViewModels
{
    public class ShowTeachersViewModel : ViewModelBase
    {
        private List<Teacher> _teachers;

        public List<Teacher> Teachers
        {
            get => _teachers;
            set => this.RaiseAndSetIfChanged(ref _teachers, value);
        }

        public ShowTeachersViewModel()
        {
            ApplyingFilters();
        }

        public async Task DeleteTeacher(Teacher teacherWasDelete)
        {
            bool queshion = await MainWindowViewModel.CusmoMSB("    ?");

            if (queshion)
            {
                MainWindowViewModel.database.Teachers.Remove(teacherWasDelete);
                MainWindowViewModel.database.SaveChanges();
                MainWindowViewModel.Instance.Page = new ShowTeachers();

                MainWindowViewModel.CusmoMSBforApprove("  !");
            }
        }

        public void GoToAddTeacher()
        {
            MainWindowViewModel.Instance.Page = new AddTeacher();
        }

        public void GoToUpdateTeacher(Teacher teacherWasUpdate)
        {
            MainWindowViewModel.Instance.Page = new AddTeacher(teacherWasUpdate);
        }

        //  
        private static string _noKnow = " ";
        private List<string> _sortValue = new List<string>()
        {
            _noKnow,
            " ",
            " "
        };
        public List<string> SortValue { get => _sortValue; }

        //
        //
        private string _search;
        public string Search
        {
            get => _search;
            set
            {
                this.RaiseAndSetIfChanged(ref _search, value);
                ApplyingFilters();
            }
        }

        //
        private List<Gender> _genderFiltr = [new Gender() { Id = 0, Gender1 = _noKnow }, .. MainWindowViewModel.database.Genders.ToList()];
        public List<Gender> GenderFiltr { get => _genderFiltr; }
        private Gender _selectedGender = null;
        public Gender SelectedGender
        {
            get
            {
                if (_selectedGender == null) return GenderFiltr[0];
                else
                {
                    return _selectedGender;
                }
            }
            set
            {
                this.RaiseAndSetIfChanged(ref _selectedGender, value);
                ApplyingFilters();
            }
        }

        // 
        private List<Lesson> _lessonFiltr = [new Lesson() { Id = 0, Lesson1 = _noKnow }, .. MainWindowViewModel.database.Lessons.ToList()];
        public List<Lesson> LessonFiltr { get => _lessonFiltr; }
        private Lesson _selectedLesson = null;
        public Lesson SelectedLesson
        {
            get
            {
                if (_selectedLesson == null) return LessonFiltr[0];
                else
                {
                    return _selectedLesson;
                }
            }
            set
            {
                this.RaiseAndSetIfChanged(ref _selectedLesson, value);
                ApplyingFilters();
            }
        }

        //  
        private List<string> _phoneExistsList = new List<string>()
        {
           _noKnow,
           "",
           ""
        };
        public List<string> PhoneExistsList { get => _phoneExistsList; }
        private int _phoneExists = 0;
        public int PhoneExists
        {
            get => _phoneExists;
            set
            {
                this.RaiseAndSetIfChanged(ref _phoneExists, value);
                ApplyingFilters();
            }
        }

        //
        //   
        private int _sortIndexCountHours = 0;
        public int SortIndexCountHours
        {
            get => _sortIndexCountHours;
            set
            {
                this.RaiseAndSetIfChanged(ref _sortIndexCountHours, value);
                ApplyingFilters();
            }
        }

        // 
        private int _sortIndexExpirience = 0;
        public int SortIndexExpirience
        {
            get => _sortIndexExpirience;
            set
            {
                this.RaiseAndSetIfChanged(ref _sortIndexExpirience, value);
                ApplyingFilters();
            }
        }

        // 
        private int _sortIndexSurname = 0;
        public int SortIndexSurname
        {
            get => _sortIndexSurname;
            set
            {
                this.RaiseAndSetIfChanged(ref _sortIndexSurname, value);
                ApplyingFilters();
            }
        }

        private void ApplyingFilters()
        {
            Teachers = MainWindowViewModel.database.Teachers
                            .Include(gen => gen.Gender)
                            .Include(course => course.TeacherCourses)
                            .ThenInclude(course => course.CourseNavigation)
                            .Include(lesson => lesson.TeacherLessons)
                            .ThenInclude(lesson => lesson.LessonNavigation)
                            .ToList();

            //  
            if (!string.IsNullOrWhiteSpace(Search)) Teachers = Teachers.Where(x => x.FIO.ToLower().Contains(Search.ToLower())).ToList();
            // 
            if (SelectedGender != null && SelectedGender.Id != 0) Teachers = Teachers.Where(x => x.Gender == SelectedGender).ToList();
            //  
            if (PhoneExists != 0) Teachers = Teachers.Where(x =>
                string.IsNullOrEmpty(x.Phone) == (_phoneExists == 1 ? false : true))
                .ToList();
            //  
            if (SortIndexCountHours != 0)
            {
                switch (SortIndexCountHours)
                {
                    case 1:
                        Teachers = Teachers.OrderBy(x => x.SumCoursesHourse).ToList();
                        break;
                    case 2:
                        Teachers = Teachers.OrderByDescending(x => x.SumCoursesHourse).ToList();
                        break;
                }
            }
            //  
            if (SelectedLesson != null && SelectedLesson.Id != 0) Teachers = Teachers.Where(x => x.TeacherLessons.Any(y => y.Lesson == SelectedLesson.Id)).ToList();
            // 
            if (SortIndexExpirience != 0)
            {
                switch (SortIndexExpirience)
                {
                    case 1:
                        Teachers = Teachers.OrderBy(x => x.Experience).ToList();
                        break;
                    case 2:
                        Teachers = Teachers.OrderByDescending(x => x.Experience).ToList();
                        break;
                }
            }
            // 
            if (SortIndexSurname != 0)
            {
                switch (SortIndexSurname)
                {
                    case 1:
                        Teachers = Teachers.OrderBy(x => x.Surname).ToList();
                        break;
                    case 2:
                        Teachers = Teachers.OrderByDescending(x => x.Surname).ToList();
                        break;
                }
            }
        }

        //     
        public void CancelingSorting()
        {
            Search = "";
            SelectedGender = null;
            SelectedLesson = null;
            PhoneExists = 0;
            SortIndexCountHours = 0;
            SortIndexExpirience = 0;
            SortIndexSurname = 0;
        }
    }
}